import sys, network_map
from pyroc import *
from collections import defaultdict

network_prefix = sys.argv[1]
eval_threshold = float(sys.argv[2])

def run_experiment(method, directed, tagline):
   global network_prefix, eval_threshold, entity1_entity2, keys, ycn_auc
   sys.stderr.write("Calculating %s..." % tagline)
   if directed:
      edges = method(entity1_entity2, keys, directed = directed)
   else:
      edges = method(entity1_entity2, keys)
   f = open("%s_%s_edges" % (network_prefix, tagline), 'w')
   roc_list = []
   for edge in edges:
      if edges[edge] > 0:
         if eval_edges[edge[0]][edge[1]] >= eval_threshold:
            first_element = 1
         else:
            first_element = 0
         roc_list.append((first_element, edges[edge]))
         f.write("%s\t%s\t%1.12f\n" % (edge[0], edge[1], edges[edge]))
   f.close()
   roc = ROCData(roc_list)
   f = open("%s_%s_roc" % (network_prefix, tagline), 'w')
   for p in roc.derived_points:
      f.write("%1.5f %1.5f\n" % (p[0], p[1]))
   f.close()
   auc = roc.auc()
   if tagline == "ycn":
      sys.stderr.write(" AUC: %1.5f\n" % auc)
   else:
      sys.stderr.write(" AUC: %1.5f (%1.5f)\n" % (auc, ycn_auc - auc))
   return tagline, roc, auc

eval_edges = defaultdict(lambda : defaultdict(float))
f = open("%s_evalnet" % network_prefix, 'r')
for line in f:
   fields = line.strip().split('\t')
   if len(fields) == 3:
      if fields[0] <= fields[1]:
         eval_edges[fields[0]][fields[1]] = float(fields[2])
      else:
         eval_edges[fields[1]][fields[0]] = float(fields[2])
   else:
      if fields[0] <= fields[1]:
         eval_edges[fields[0]][fields[1]] = 1.0
      else:
         eval_edges[fields[1]][fields[0]] = 1.0
f.close()

keys = {}
entity1_entity2 = defaultdict(list)
f = open("%s_bipartite" % network_prefix, 'r')
for line in f:
   fields = line.strip().split('\t')
   if fields[0] in eval_edges:
      keys[fields[0]] = 0
      keys[fields[1]] = 1
      entity1_entity2[fields[0]].append(fields[1])
      entity1_entity2[fields[1]].append(fields[0])
f.close()

ycn_auc = 0.0
rocs = []
f = open("%s_auc" % (network_prefix), 'w')
tag, roc, auc = run_experiment(network_map.ycn_edges, False, "ycn")
ycn_auc = auc
f.write("%s %1.5f\n" % (tag, auc))
rocs.append(roc)
tag, roc, auc = run_experiment(network_map.zhou, True, "zho")
f.write("%s %1.5f\n" % (tag, auc))
rocs.append(roc)
tag, roc, auc = run_experiment(network_map.jaccard, False, "jac")
f.write("%s %1.5f\n" % (tag, auc))
rocs.append(roc)
tag, roc, auc = run_experiment(network_map.cosine, False, "cos")
f.write("%s %1.5f\n" % (tag, auc))
rocs.append(roc)
tag, roc, auc = run_experiment(network_map.euclidean, False, "euc")
f.write("%s %1.5f\n" % (tag, auc))
rocs.append(roc)
tag, roc, auc = run_experiment(network_map.pearson, False, "pea")
f.write("%s %1.5f\n" % (tag, auc))
rocs.append(roc)
f.close()
sys.stderr.write("\n")

plot_multiple_roc(rocs, 'Multiple ROC Curves', include_baseline = True)
