import sys, network_map, operator, random, copy
from collections import defaultdict

network_prefix = sys.argv[1]

nedges = 0
sys.stderr.write("Reading Network...\n")
keys = {}
entity1_entity2 = defaultdict(list)
f = open("%s_bipartite" % network_prefix, 'r')
for line in f:
   fields = line.strip().split('\t')
   keys["m%s" % fields[0]] = 0
   keys["u%s" % fields[1]] = 1
   entity1_entity2["m%s" % fields[0]].append("u%s" % fields[1])
   entity1_entity2["u%s" % fields[1]].append("m%s" % fields[0])
   nedges += 1
f.close()

toremove = nedges / 10

sys.stderr.write("Removing Random Edges...\n")
entity1_entity2_full = copy.deepcopy(entity1_entity2)
movies = [key for key in keys if keys[key] == 0]
users = [key for key in keys if keys[key] != 0]
for i in xrange(toremove):
   removed = False
   while not removed:
      user = random.choice(users)
      movie = random.choice(movies)
      if movie in entity1_entity2[user]:
         removed = True
         entity1_entity2[user].remove(movie)
         entity1_entity2[movie].remove(user)

edges = defaultdict(lambda : defaultdict(lambda : defaultdict(float)))
sys.stderr.write("Calculating ycn edges...\n")
edges["ycn"] =  network_map.ycn_edges(entity1_entity2, keys, directed = True)
sys.stderr.write("Calculating zho edges...\n")
edges["zho"] =  network_map.zhou(entity1_entity2, keys, directed = True)
sys.stderr.write("Calculating jac edges...\n")
edges["jac"] =  network_map.jaccard(entity1_entity2, keys)
sys.stderr.write("Calculating euc edges...\n")
edges["euc"] =  network_map.euclidean(entity1_entity2, keys)
sys.stderr.write("Calculating cos edges...\n")
edges["cos"] =  network_map.cosine(entity1_entity2, keys)
sys.stderr.write("Calculating pea edges...\n")
edges["pea"] =  network_map.pearson(entity1_entity2, keys)

predictions_rs = defaultdict(list)
hit_indexes = defaultdict(list)
sys.stderr.write("Calculating predictions...\n")

j = 0
toolbar_width = 100
updates = float(len(users)) / 100
sys.stderr.write("[%s]" % (" " * toolbar_width))
sys.stderr.flush()
sys.stderr.write("\b" * (toolbar_width + 1))

for user in users:
   method_moviescore = defaultdict(lambda : defaultdict(float))
   for movie in movies:
      if not movie in entity1_entity2[user]:
         for method in edges:
            for movie2 in entity1_entity2[user]:
               method_moviescore[method][movie] += edges[method][(movie2, movie)]
   for method in method_moviescore:
      usermovie_score_sorted = sorted(method_moviescore[method].iteritems(), key = operator.itemgetter(1), reverse = True)
      for movie in entity1_entity2_full[user]:
         if not movie in entity1_entity2[user]:
            for i in range(len(usermovie_score_sorted)):
               if movie == usermovie_score_sorted[i][0]:
                  hit_indexes[method].append(i)
                  predictions_rs[method].append(float(i) / len(usermovie_score_sorted))
                  break
   j += 1
   if j > updates:
      sys.stderr.write("=")
      sys.stderr.flush()
      j -= updates
sys.stderr.write("\n")

for method in method_moviescore:
   sys.stderr.write("%s's Average r: %1.5f\n" % (method, sum(predictions_rs[method]) / len(predictions_rs[method])))
   sys.stderr.write("%s's Average hit length: %1.5f\n\n" % (method, float(sum(hit_indexes[method])) / len(hit_indexes[method])))
   cur_predictions_rs = sorted(predictions_rs[method])
   f = open("%s_predictions_rs_%s" % (network_prefix, method), 'w')
   for i in range(len(cur_predictions_rs)):
      f.write("%d %1.2f\n" % (i, cur_predictions_rs[i]))
   f.close()
   cur_hit_indexes = sorted(hit_indexes[method])
   f = open("%s_hit_indexes_%s" % (network_prefix, method), 'w')
   for i in cur_hit_indexes:
      f.write("%d\n" % i)
   f.close()
